//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  dualplot.cpp
//! \brief Implements class CPlot.

#include <iostream>

#include "../trivia/string_convs.hpp"
#include "dualplot.hpp"
#include "gnuplotter.hpp"
#include "ps_plotter.hpp"

#define S(a) triv::strg((a))

using std::string;
using std::vector;
using std::cout;

//! Constructor for plot window: setup for gnuplot and postscript.

CPlot::CPlot(int _iPlot, bool _logx, bool _logy)
    : iPlot(_iPlot)
    , X("x", _logx)
    , Y("y", _logy)
    , maxpoints(20000)
    , with_errors(true)
    , equipoints(120)
    , refine(true)
    , gnuPlotter(new CGnuPlotter(iPlot))
    , ps_Plotter(new CPS_Plotter)
{}


//! Clear plot frame and buffer.

void CPlot::clear_frame()
{
    gnuPlotter->clear();
    ps_Plotter->clear();
}


//! Change one setup parameter per command.

void CPlot::set_aux(const string& cmd)
{
    if (cmd == "gxl") {
        X.set_log(!X.logflag);
        printf("set x %s\n", X.logflag ? "log" : "lin");
    } else if (cmd == "gxl+")
        X.set_log(true);
    else if (cmd == "gxl-")
        X.set_log(false);
    else if (cmd == "gyl") {
        Y.set_log(!Y.logflag);
        printf("set y %s\n", Y.logflag ? "log" : "lin");
    } else if (cmd == "gyl+")
        Y.set_log(true);
    else if (cmd == "gyl-")
        Y.set_log(false);
    else if (cmd == "gxf") {
        X.force = !X.force;
        printf("force x %s\n", X.force ? "on" : "off");
    } else if (cmd == "gxf+")
        X.force = true;
    else if (cmd == "gxf-")
        X.force = false;
    else if (cmd == "gyf") {
        Y.force = !Y.force;
        printf("force y %s\n", Y.force ? "on" : "off");
    } else if (cmd == "gyf+")
        Y.force = true;
    else if (cmd == "gyf-")
        Y.force = false;
    else if (cmd == "ge")
        with_errors = !with_errors;
    else if (cmd == "ge+")
        with_errors = true;
    else if (cmd == "ge-")
        with_errors = false;
    else
        throw "unknown command " + cmd;
}


//! Plot coordinate frame (axes, ticks, labels).

void CPlot::plot_frame(const string& xlabel, const string& ylabel)
{
    gnuPlotter->plot_frame(X, Y);
    ps_Plotter->plot_frame(X, Y, xlabel, ylabel);
}


//! Plot one spectrum.

void CPlot::add_spec(
    bool as_line, bool new_slice, int style_no, const vector<double>& xp, const vector<double>& yp,
    const vector<double>& dyp, const vector<string>& zentries,
    const string& xco, const string& yco, const string& info)
{
    gnuPlotter->add_spec(
        as_line, new_slice, with_errors,
        style_no, xp, yp, dyp, zentries, xco, yco, info);
    ps_Plotter->add_spec(
        as_line, new_slice,
        style_no, xp, yp, dyp, zentries, xco, yco, info);
}

void CPlot::show_specs()
{
    gnuPlotter->show_specs();
}

//! Add documentation line to postscript output.

void CPlot::doc_TxLine(const string& line)
{
    ps_Plotter->doc_line("  {(" + line + ")} TxLine");
}


//! Add documentation line explaining a plot symbol to postscript output.

void CPlot::doc_PtTxLine(const string& line, int num)
{
    ps_Plotter->doc_line("  " + S(num) + " {(" + line + ")} PtTxLine");
}


//! Add documentation line explaining a curve style to postscript output.

void CPlot::doc_CvTxLine(const string& line, int num)
{
    ps_Plotter->doc_line("  " + S(num) + " {(" + line + ")} CvTxLine");
}


//! Write buffered plot to postscript file.

void CPlot::write_postscript(
    const string& mode, const string& fname, const string& ps_outdir,
    const string& ps_head, const string& ps_cont, const string& ps_dict)
{
    ps_Plotter->write_postscript(mode, fname, ps_outdir, ps_head, ps_cont, ps_dict);
}

//! Info line to characterize this plot window.

string CPlot::info() const
{
    return "x: " + X.info() +  "  y: " + Y.info();
}


//! Send one line to gnuplot fifo.

void CPlot::gp_write(const string& in)
{
    gnuPlotter->write(in);
}
