//****************************************************************************//
//* libtrivia: various little routines for C++                               *//
//* rng: random number generator wrapper                                     *//
//* (C) Joachim Wuttke, Antti Soininen 2001, 2012, 2013                      *//
//* http://apps.jcns.fz-juelich.de                                           *//
//****************************************************************************//

#include <chrono>
#include <gsl/gsl_randist.h>
#include <math.h>

#include "rng.hpp"

//****************************************************************************//
//  Implementation class
//****************************************************************************//

class triv::RandomNumberImplementation
{
public:
    RandomNumberImplementation() { r = gsl_rng_alloc(gsl_rng_default); }
    ~RandomNumberImplementation() { gsl_rng_free(r); }
    void reseed(unsigned long seed) { gsl_rng_set(r, seed); }
    double uniform() { return gsl_rng_uniform(r); }
    double gaussian(double stdv) { return gsl_ran_gaussian(r, stdv); }
    double cauchy(double width) { return gsl_ran_cauchy(r, width); }
private:
    gsl_rng* r;
};

//****************************************************************************//
//  Interface class
//****************************************************************************//

triv::RandomNumberGenerator::RandomNumberGenerator()
{ pimpl = new triv::RandomNumberImplementation(); }

triv::RandomNumberGenerator::~RandomNumberGenerator() { delete pimpl; }

//! Set new random seed.

void triv::RandomNumberGenerator::reseed(unsigned long seed) { pimpl->reseed(seed); }

//! Set new random seed from clock's nsec.

void triv::RandomNumberGenerator::reseed_from_clock()
{
    std::chrono::nanoseconds time = std::chrono::system_clock::now().time_since_epoch();
    reseed((unsigned long)time.count());
}

double triv::RandomNumberGenerator::uniform() { return pimpl->uniform(); }

double triv::RandomNumberGenerator::uniform(double rmin, double rmax)
{
    return rmin + (rmax - rmin) * uniform();
}

double triv::RandomNumberGenerator::gaussian(double stdv, double cutoff)
{
    double ret;
    do {
        ret = pimpl->gaussian(stdv);
    } while (cutoff && fabs(ret) > cutoff);
    return ret;
}

double triv::RandomNumberGenerator::cauchy(double width, double cutoff)
{
    double ret;
    do {
        ret = pimpl->cauchy(width);
    } while (cutoff && fabs(ret) > cutoff);
    return ret;
}
